/***************************************************************
	Populate cache
***************************************************************/

USE Northwind
GO
EXEC [dbo].[Sales by Year] '1996','1997'
EXEC [dbo].[Sales by Year] '1997','1998'
EXEC [dbo].[Sales by Year] '1998','1999'

EXEC [dbo].[Ten Most Expensive Products] 

EXEC [dbo].[Employee Sales by Country] '1996','1997'
EXEC [dbo].[Employee Sales by Country] '1997','1998'
EXEC [dbo].[Employee Sales by Country] '1998','1999'

EXEC [dbo].[CustOrderHist] 'ALFKI'
EXEC [dbo].[CustOrderHist] 'ANATR'
EXEC [dbo].[CustOrderHist] 'ANTON'
EXEC [dbo].[CustOrderHist] 'AROUT'
EXEC [dbo].[CustOrderHist] 'BERGS'
EXEC [dbo].[CustOrderHist] 'BLAUS'
EXEC [dbo].[CustOrderHist] 'ALFKI'
EXEC [AdventureWorks2012].[dbo].[uspLogError]
EXEC [AdventureWorks2012].[dbo].[uspGetManagerEmployees] 1
EXEC [AdventureWorks2012].[dbo].[uspGetManagerEmployees] 2
EXEC [AdventureWorks2012].[dbo].[uspGetEmployeeManagers] 1
EXEC [AdventureWorks2012].[dbo].[uspGetEmployeeManagers] 2
EXEC [AdventureWorks2012].[dbo].[uspGetEmployeeManagers] 3
EXEC [AdventureWorks2012].[dbo].[uspGetEmployeeManagers] 4
EXEC [AdventureWorks2012].[dbo].[uspPrintError] 

DECLARE @OrderDate DATETIME
SET @OrderDate = '20040731'
SELECT * FROM AdventureWorks2012.Sales.SalesOrderHeader h, 
AdventureWorks2012.Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderID
AND h.OrderDate = @OrderDate

/***************************************************************
Fill cache and show memory fingerprint
***************************************************************/
USE Northwind
GO
IF EXISTS(SELECT * FROM SYS.TABLES WHERE name = 'SQLBits')
	DROP TABLE SQLBits
GO
CREATE TABLE SQLBits(session_id SMALLINT)

DECLARE @i int = 1
DECLARE @cmd varchar(8000)
WHILE @i < 5000
BEGIN
SET @cmd = '
	insert SQLBits
	select top 1 s.session_id 
	from sys.dm_exec_sessions s inner join sys.dm_exec_connections c
	on s.session_id = c.session_id
	where s.session_id = ' + CAST(@i AS VARCHAR(100))
	EXEC(@cmd)
	SET @i+=1
END



/***************************************************************
	Memory cost - Show memory fingerprint of the 5000 commands
***************************************************************/
SELECT SUM(size_in_bytes)/1024.0/1024.0
FROM sys.dm_exec_cached_plans
CROSS APPLY sys.dm_exec_sql_text(plan_handle)
WHERE text LIKE '%insert SQLBits%'



/***************************************************************
	sys.dm_exec_cached_plans
***************************************************************/
SELECT * FROM sys.dm_exec_cached_plans



/***************************************************************
	sys.dm_exec_query_stats
***************************************************************/
SELECT * FROM sys.dm_exec_query_stats



/***************************************************************
	sys.dm_exec_sql_text and sys.dm_exec_query_plan
***************************************************************/

SELECT 
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
t.dbid,
t.objectid,
p.query_plan,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_query_plan(plan_handle) p


/***************************************************************
	sys.dm_exec_text_query_plan
***************************************************************/
USE Northwind
GO
--CLEAR CACHE
DBCC FREEPROCCACHE
--RUN THE THREE SELECT STATEMENTS TOGETHER

SELECT * FROM Products
SELECT MAX(UnitPrice) FROM Products 
SELECT p.ProductName, s.CompanyName
FROM Products p
INNER JOIN Suppliers s
ON p.SupplierID = s.SupplierID

--SHOW EXECUTION PLAN WITH sys.dm_exec_query_plan
SELECT 
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
p.query_plan,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_query_plan(plan_handle) p
WHERE text LIKE '%SELECT * FROM Products%'
AND text NOT LIKE '%SUBSTRING%'

--NOW SHOW EXECUTION PLAN WITH sys.dm_exec_text_query_plan
SELECT 
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
CAST(p.query_plan AS XML) AS query_plan,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_text_query_plan(plan_handle, statement_start_offset, statement_end_offset) p
WHERE text LIKE '%SELECT * FROM Products%'
AND text NOT LIKE '%SUBSTRING%'




/***************************************************************
	Ad-hoc lookup
***************************************************************/
USE Northwind
GO
--CLEAR CACHE
DBCC FREEPROCCACHE
GO
--RUN THE THREE SELECT STATEMENTS

SELECT FirstName FROM Employees WHERE LastName=N'Davolio' AND PostalCode=N'98122'
GO
SELECT FirstName FROM Employees 
	WHERE LastName=N'Davolio' AND PostalCode=N'98122'
GO
SELECT FirstName 
FROM Employees 
WHERE LastName=N'Davolio' AND PostalCode=N'98122'
GO

SELECT c.objtype, c.plan_handle, c.usecounts, c.size_in_bytes , text
		FROM sys.dm_exec_cached_plans c
		CROSS APPLY sys.dm_exec_sql_text (plan_handle) 
		WHERE text LIKE '%SELECT FirstName%'
		AND text NOT LIKE '%c.cacheobjtype%'

/***************************************************************
	Look for Ad-hoc plans with use count = 1 and show memory
***************************************************************/
DBCC FREEPROCCACHE
GO
USE Northwind
GO
DECLARE @i int = 1
DECLARE @cmd varchar(8000)
WHILE @i < 5000
BEGIN
SET @cmd = '
	insert SQLBits
	select top 1 s.session_id 
	from sys.dm_exec_sessions s inner join sys.dm_exec_connections c
	on s.session_id = c.session_id
	where s.session_id = ' + CAST(@i AS VARCHAR(100))
	EXEC(@cmd)
	SET @i+=1
END
GO

SELECT 
SUM(CAST(size_in_bytes AS BIGINT))/1024.0/1024.0 
AS SizeInMB, COUNT(*) AS PlanCount
FROM sys.dm_exec_cached_plans
WHERE objtype = 'Adhoc' 
AND cacheobjtype = 'Compiled Plan' 
AND usecounts = 1
GO
SELECT c.* FROM sys.dm_exec_cached_plans c
CROSS APPLY sys.dm_exec_sql_text(plan_handle)
WHERE text LIKE '%insert SQLBits%'



/***************************************************************
	Set optimize for ad hoc workloads on
***************************************************************/
exec sp_configure 'show advanced options', 1
RECONFIGURE
GO
exec sp_configure 'optimize for ad hoc workloads', 1
RECONFIGURE
/***************************************************************
	Code from video to fill cache and show memory fingerprint
***************************************************************/
--CLEAR CACHE
DBCC FREEPROCCACHE
GO
USE Northwind
GO

DECLARE @i int = 1
DECLARE @cmd varchar(8000)
WHILE @i < 5000
BEGIN
SET @cmd = '
	insert SQLBits
	select top 1 s.session_id 
	from sys.dm_exec_sessions s inner join sys.dm_exec_connections c
	on s.session_id = c.session_id
	where s.session_id = ' + CAST(@i AS VARCHAR(100))
	EXEC(@cmd)
	SET @i+=1
END


/***************************************************************
	Show memory fingerprint of the 5000 commands
***************************************************************/
SELECT SUM(size_in_bytes)/1024.0/1024.0 AS SizeInMB
FROM sys.dm_exec_cached_plans
CROSS APPLY sys.dm_exec_sql_text(plan_handle)
WHERE text LIKE '%insert SQLBits%'
GO
SELECT c.* FROM sys.dm_exec_cached_plans c
CROSS APPLY sys.dm_exec_sql_text(plan_handle)
WHERE text LIKE '%insert SQLBits%'


/***************************************************************
	Set optimize for ad hoc workloads off
***************************************************************/
exec sp_configure 'optimize for ad hoc workloads', 0
RECONFIGURE


/***************************************************************
	Parameterized method: 
***************************************************************/
DBCC FREEPROCCACHE

USE Northwind
GO

DECLARE @i int = 1
DECLARE @cmd nvarchar(4000)
WHILE @i < 5000
BEGIN
SET @cmd = '
	insert SQLBits
	select top 1 s.session_id 
	from sys.dm_exec_sessions s inner join sys.dm_exec_connections c
	on s.session_id = c.session_id
	where s.session_id = @session_id' 
	EXEC sp_executesql @cmd, N'@session_id int', @session_id = @i
	SET @i+=1
END
GO

/***************************************************************
	Show how the cache looks
***************************************************************/
SELECT SUM(size_in_bytes)/1024.0/1024.0
FROM sys.dm_exec_cached_plans
CROSS APPLY sys.dm_exec_sql_text(plan_handle)
WHERE text LIKE '%insert SQLBits%'
GO
SELECT c.* FROM sys.dm_exec_cached_plans c
CROSS APPLY sys.dm_exec_sql_text(plan_handle)
WHERE text LIKE '%insert SQLBits%'
AND text NOT LIKE '%sys.dm_exec_cached_plans%'

/***************************************************************
	Parameterized queries lack of reuse
***************************************************************/
--CLEAR CACHE
DBCC FREEPROCCACHE
GO
USE Northwind
GO

--RUN THE TWO STATEMENTS
EXEC sp_executesql N'SELECT FirstName FROM Employees WHERE LastName=@LastName AND PostalCode=@PostalCode'
	,N'@LastName NVARCHAR(7),@PostalCode NVARCHAR(5)',@LastName = N'Davolio',@PostalCode = N'98122'
GO
EXEC sp_executesql N'SELECT FirstName FROM Employees WHERE LastName=@LastName AND PostalCode=@PostalCode'
	,N'@LastName NVARCHAR(6),@PostalCode NVARCHAR(5)',@LastName = N'Fuller',@PostalCode = N'98401'

SELECT t.text, c.* FROM sys.dm_exec_cached_plans c
CROSS APPLY sys.dm_exec_sql_text(plan_handle) t
WHERE text LIKE '%SELECT FirstName FROM Employees%'
AND text NOT LIKE '%SELECT t.text%'

--THE WAY TO SOLVE IT:
DBCC FREEPROCCACHE
GO
EXEC sp_executesql N'SELECT FirstName FROM Employees WHERE LastName=@LastName AND PostalCode=@PostalCode'
	,N'@LastName NVARCHAR(40),@PostalCode NVARCHAR(20)',@LastName = N'Davolio',@PostalCode = N'98122'
GO
EXEC sp_executesql N'SELECT FirstName FROM Employees WHERE LastName=@LastName AND PostalCode=@PostalCode'
	,N'@LastName NVARCHAR(40),@PostalCode NVARCHAR(20)',@LastName = N'Fuller',@PostalCode = N'98401'


SELECT t.text, c.* FROM sys.dm_exec_cached_plans c
CROSS APPLY sys.dm_exec_sql_text(plan_handle) t
WHERE text LIKE '%SELECT FirstName FROM Employees%'
AND text NOT LIKE '%SELECT t.text%'



/***************************************************************
	Populate cache 
***************************************************************/
USE Northwind
GO
EXEC PopulateCache

/***************************************************************
	Detecting problematic queries
***************************************************************/
--Top queries by execution logical_reads
SELECT TOP 20
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
CAST(p.query_plan AS XML) AS query_plan,
db_name(t.dbid) AS DatabaseName,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_text_query_plan(plan_handle, statement_start_offset, statement_end_offset) p
ORDER BY s.total_logical_reads DESC



--Top queries by execution duration
SELECT TOP 20
s.total_elapsed_time/CAST(s.execution_count AS DECIMAL)/1000000.0 AS elapsed_time_per_execution_seconds,
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
CAST(p.query_plan AS XML) AS query_plan,
db_name(t.dbid) AS DatabaseName,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_text_query_plan(plan_handle, statement_start_offset, statement_end_offset) p
ORDER BY s.total_elapsed_time/CAST(s.execution_count AS DECIMAL) DESC



--Top resource intensive query "families"
SELECT TOP 10
   query_hash,sum(s.total_elapsed_time) AS sum_elapsed_time
   FROM sys.dm_exec_query_stats s
   GROUP BY query_hash
   ORDER BY sum_elapsed_time DESC

--Take query_hash and put in the Where
SELECT
s.total_elapsed_time/CAST(s.execution_count AS DECIMAL) AS elapsed_time_per_execution,
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
CAST(p.query_plan AS XML) AS query_plan,
db_name(t.dbid) AS DatabaseName,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_text_query_plan(plan_handle, statement_start_offset, statement_end_offset) p
WHERE query_hash = 0xF9B713507F303B98



--Most resource intensive database
SELECT DB_NAME(t.dbid) AS DBName, SUM(total_logical_reads + total_logical_writes) AS TotalIO
FROM sys.dm_exec_query_stats
CROSS APPLY sys.dm_exec_sql_text (plan_handle) t
GROUP BY DB_NAME(t.dbid)
ORDER BY TotalIO DESC

--Find the value a stored procedure was optimized for (inside the execution plan xml)
USE Northwind
GO
SELECT
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
CAST(p.query_plan AS XML) AS query_plan,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
cross apply sys.dm_exec_query_plan(plan_handle) p
WHERE t.objectid = object_id('CustOrderHist')

--Top queries by execution plan_generation_num
SELECT TOP 20
plan_generation_num,
SUBSTRING(text, statement_start_offset/2+1, 
		 ((CASE WHEN statement_end_offset = -1 THEN DATALENGTH(text) 
		 ELSE statement_end_offset 
		 END - statement_start_offset)/2) + 1) AS running_statement,
text AS current_batch,
CAST(p.query_plan AS XML) AS query_plan,
db_name(t.dbid) AS DatabaseName,
s.*
FROM sys.dm_exec_query_stats s
cross apply sys.dm_exec_sql_text(sql_handle) t
outer apply sys.dm_exec_text_query_plan(plan_handle, statement_start_offset, statement_end_offset) p
ORDER BY s.plan_generation_num DESC


/***************************************************************
	Querying Execution Plan XML
***************************************************************/

--Look for queries that have specific problematic operators
IF EXISTS(SELECT * FROM sys.procedures WHERE name = 'LookForPhysicalOps')
	DROP PROCEDURE LookForPhysicalOps
GO
--Credit: Bob Beauchemin - http://www.sqlskills.com/blogs/bobb/move-over-developers-sql-server-xquery-is-actually-a-dba-tool/
CREATE PROCEDURE LookForPhysicalOps (@op VARCHAR(30))
AS
SELECT sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE query_plan.exist('
declare default element namespace "http://schemas.microsoft.com/sqlserver/2004/07/showplan";
/ShowPlanXML/BatchSequence/Batch/Statements//RelOp/@PhysicalOp[. = sql:variable("@op")]
') = 1
GO

EXECUTE LookForPhysicalOps 'Clustered Index Scan'
EXECUTE LookForPhysicalOps 'Hash Match'
EXECUTE LookForPhysicalOps 'Table Scan'
EXECUTE LookForPhysicalOps 'Sort'

GO

--Adjusting the procedure a bit in order to return only those that are in the top 20 by IO/Time:
IF EXISTS(SELECT * FROM sys.procedures WHERE name = 'LookForPhysicalOps_Adjusted')
	DROP PROCEDURE LookForPhysicalOps_Adjusted
GO
CREATE PROCEDURE LookForPhysicalOps_Adjusted (@op VARCHAR(30), @OrderBy VARCHAR(10))
AS
SELECT 
sql.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
CROSS APPLY sys.dm_exec_sql_text(sql_handle) sql
CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
INNER JOIN --In order to return only those that are in the top 20
	(SELECT TOP 20 plan_handle, statement_start_offset, statement_end_offset FROM sys.dm_exec_query_stats
	 ORDER BY 
	CASE WHEN @OrderBy = 'IO' THEN total_logical_reads 
		 WHEN @OrderBy = 'Time' THEN total_elapsed_time 
		 END
	DESC) AS Top20
ON qs.plan_handle = Top20.plan_handle 
AND qs.statement_start_offset = Top20.statement_start_offset 
AND qs.statement_end_offset = Top20.statement_end_offset	 
WHERE query_plan.exist('
declare default element namespace "http://schemas.microsoft.com/sqlserver/2004/07/showplan";
/ShowPlanXML/BatchSequence/Batch/Statements//RelOp/@PhysicalOp[. = sql:variable("@op")]
') = 1


--Showing sorts that are in the top 20 by IO
EXEC LookForPhysicalOps_Adjusted 'Sort', 'IO'
--Showing sorts that are in the top 20 by Time
EXEC LookForPhysicalOps_Adjusted 'Sort', 'Time'
GO


/***************************************************************
	We can also query the XML as nvarchar(max)
***************************************************************/

--Finding plans with warnings that are in the top 20 queries by IO:
SELECT  DB_NAME(detqp.dbid), 
        SUBSTRING(dest.text, (deqs.statement_start_offset / 2) + 1, 
                  (CASE deqs.statement_end_offset 
                     WHEN -1 THEN DATALENGTH(dest.text) 
                     ELSE deqs.statement_end_offset 
                   END - deqs.statement_start_offset) / 2 + 1) AS StatementText, 
        CAST(detqp.query_plan AS XML), 
        deqs.execution_count, 
        deqs.total_elapsed_time, 
        deqs.total_logical_reads, 
        deqs.total_logical_writes 
FROM    sys.dm_exec_query_stats AS deqs 
        CROSS APPLY sys.dm_exec_text_query_plan(deqs.plan_handle, 
                                                deqs.statement_start_offset, 
                                                deqs.statement_end_offset) AS detqp 
        CROSS APPLY sys.dm_exec_sql_text(deqs.sql_handle) AS dest 
		INNER JOIN --In order to return only those that are in the top 20
	(SELECT TOP 20 plan_handle, statement_start_offset, statement_end_offset FROM sys.dm_exec_query_stats
	ORDER BY total_logical_reads DESC
	) AS Top20
	ON deqs.plan_handle = Top20.plan_handle 
	AND deqs.statement_start_offset = Top20.statement_start_offset 
	AND deqs.statement_end_offset = Top20.statement_end_offset	 
WHERE   detqp.query_plan LIKE '%<Warnings>%';



-- Finding plans that their optimizations were timed-out:

--Credit: Grant Fritchy - http://www.scarydba.com/2012/07/02/querying-data-from-the-plan-cache/
SELECT  DB_NAME(detqp.dbid), 
        SUBSTRING(dest.text, (deqs.statement_start_offset / 2) + 1, 
                  (CASE deqs.statement_end_offset 
                     WHEN -1 THEN DATALENGTH(dest.text) 
                     ELSE deqs.statement_end_offset 
                   END - deqs.statement_start_offset) / 2 + 1) AS StatementText, 
        CAST(detqp.query_plan AS XML), 
        deqs.execution_count, 
        deqs.total_elapsed_time, 
        deqs.total_logical_reads, 
        deqs.total_logical_writes 
FROM    sys.dm_exec_query_stats AS deqs 
        CROSS APPLY sys.dm_exec_text_query_plan(deqs.plan_handle, 
                                                deqs.statement_start_offset, 
                                                deqs.statement_end_offset) AS detqp 
        CROSS APPLY sys.dm_exec_sql_text(deqs.sql_handle) AS dest 
WHERE   detqp.query_plan LIKE '%StatementOptmEarlyAbortReason="TimeOut"%';


--Finding plans with non parallel plan warnings that are in the top 20 queries by duration:
SELECT  DB_NAME(detqp.dbid), 
        SUBSTRING(dest.text, (deqs.statement_start_offset / 2) + 1, 
                  (CASE deqs.statement_end_offset 
                     WHEN -1 THEN DATALENGTH(dest.text) 
                     ELSE deqs.statement_end_offset 
                   END - deqs.statement_start_offset) / 2 + 1) AS StatementText, 
        CAST(detqp.query_plan AS XML), 
        deqs.execution_count, 
        deqs.total_elapsed_time, 
        deqs.total_logical_reads, 
        deqs.total_logical_writes 
FROM    sys.dm_exec_query_stats AS deqs 
        CROSS APPLY sys.dm_exec_text_query_plan(deqs.plan_handle, 
                                                deqs.statement_start_offset, 
                                                deqs.statement_end_offset) AS detqp 
        CROSS APPLY sys.dm_exec_sql_text(deqs.sql_handle) AS dest 
		INNER JOIN --In order to return only those that are in the top 20
	(SELECT TOP 20 plan_handle, statement_start_offset, statement_end_offset FROM sys.dm_exec_query_stats
	ORDER BY total_elapsed_time DESC
	) AS Top20
	ON deqs.plan_handle = Top20.plan_handle 
	AND deqs.statement_start_offset = Top20.statement_start_offset 
	AND deqs.statement_end_offset = Top20.statement_end_offset	 
WHERE   detqp.query_plan LIKE '%NonParallelPlanReason%';

/******************************************
Other Reasons - Zoomit
******************************************/
/*
CLRUserDefinedFunctionRequiresDataAccess
CouldNotGenerateValidParallelPlan
DMLQueryReturnsOutputToClient
EstimatedDOPIsOne
MaxDOPSetToOne
MixedSerialAndParallelOnlineIndexBuildNotSupported
NonParallelizableIntrinsicFunction
NoParallelCreateIndexInNonEnterpriseEdition
NoParallelCursorFetchByBookmark
NoParallelDynamicCursor
NoParallelFastForwardCursor
NoParallelForCloudDBReplication
NoParallelForPDWCompilation
NoParallelHekatonPlan
NoParallelPlansDuringUpgrade
NoParallelPlansInDesktopOrExpressEdition
NoRemoteParallelismForMatrix
ParallelismDisabledByTraceFlag
TableVariableTransactionsDoNotSupportParallelNestedTransaction
TSQLUserDefinedFunctionsNotParallelizable
UNUSED
UpdatingWritebackVariable
*/


--Finding plans with missing index that are in the top 20 queries by IO:
SELECT  DB_NAME(detqp.dbid), 
        SUBSTRING(dest.text, (deqs.statement_start_offset / 2) + 1, 
                  (CASE deqs.statement_end_offset 
                     WHEN -1 THEN DATALENGTH(dest.text) 
                     ELSE deqs.statement_end_offset 
                   END - deqs.statement_start_offset) / 2 + 1) AS StatementText, 
        CAST(detqp.query_plan AS XML), 
        deqs.execution_count, 
        deqs.total_elapsed_time, 
        deqs.total_logical_reads, 
        deqs.total_logical_writes 
FROM    sys.dm_exec_query_stats AS deqs 
        CROSS APPLY sys.dm_exec_text_query_plan(deqs.plan_handle, 
                                                deqs.statement_start_offset, 
                                                deqs.statement_end_offset) AS detqp 
        CROSS APPLY sys.dm_exec_sql_text(deqs.sql_handle) AS dest 
		INNER JOIN --In order to return only those that are in the top 20
	(SELECT TOP 20 plan_handle, statement_start_offset, statement_end_offset FROM sys.dm_exec_query_stats
	ORDER BY total_logical_reads DESC
	) AS Top20
	ON deqs.plan_handle = Top20.plan_handle 
	AND deqs.statement_start_offset = Top20.statement_start_offset 
	AND deqs.statement_end_offset = Top20.statement_end_offset	 
WHERE   detqp.query_plan LIKE '%MissingIndexGroup%';


--Detecting row count estimation problems
--Based on a post by Joe Sack:
--http://www.sqlskills.com/blogs/joe/detecting-cardinality-estimate-issues-with-sys-dm_exec_query_stats/

--Make some cardinality estimate fault:
DECLARE @OrderDate DATETIME
SET @OrderDate = '20040731'
SELECT * FROM AdventureWorks2012.Sales.SalesOrderHeader h, 
AdventureWorks2012.Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderID
AND h.OrderDate = @OrderDate



SELECT TOP 100 *
FROM
(
SELECT
 t.text as [batch_text],
 SUBSTRING(t.text, statement_start_offset/2+1,
 ((case when statement_end_offset = -1
 then datalength(text)
 else statement_end_offset end
 - statement_start_offset)/2) + 1) as [query_text],
 CAST(p.[query_plan] AS XML) AS [query_plan],
 s.[last_execution_time],
 S.[last_elapsed_time],
 CAST(p.[query_plan] AS XML).value('(//@EstimateRows)[1]', 'VARCHAR(128)') AS [estimated_rows],
 s.[last_rows]
FROM sys.[dm_exec_query_stats] AS [s]
CROSS APPLY sys.[dm_exec_sql_text]
        (sql_handle) AS [t]
CROSS APPLY sys.[dm_exec_text_query_plan]
        (plan_handle,statement_start_offset,statement_end_offset) AS [p]
) AS InnerQuery
WHERE TRY_CONVERT(DECIMAL,InnerQuery.[estimated_rows]) IS NOT NULL
AND ABS(CONVERT(DECIMAL,InnerQuery.[estimated_rows]) - [last_rows]) > 300
ORDER BY ABS(CONVERT(DECIMAL,InnerQuery.[estimated_rows]) - [last_rows]) DESC



/***************************************************************
	All done
***************************************************************/